@@ -1,8 +1,6 @@
 package YAML::LoadBundle;
 # ABSTRACT: Load a directory of YAML files as a bundle
-use version;
-our $VERSION = 'v0.4.3'; # VERSION
-
+$YAML::LoadBundle::VERSION = '0.001';
 use base qw(Exporter);
 use warnings;
 use strict;
@@ -13,8 +11,10 @@ use Digest::SHA1        qw( sha1_hex sha1 );
 use File::Find          qw( find );
 use Hash::Merge::Simple ();
 use Scalar::Util        qw( reftype refaddr );
-use Storable            qw( freeze dclone );
-use YAML::XS            qw(Load);
+use Storable            qw( freeze );
+
+use YAML::XS qw(Load);
+#use Data::Visitor::Callback;
 
 our @EXPORT_OK = qw(
     load_yaml
@@ -52,18 +52,12 @@ sub remove_yaml_observer {
     } @load_yaml_observers;
 }
 
-our %seen;
-
 sub load_yaml {
     my ($arg, $dont_cache) = @_;
     my @yaml;
     my $cache_mtime;
     my %params;
 
-    # We clone references that appear more than once in the data
-    # structure. (For compatibility with Data::Visitor.)
-    local %seen = ();
-
     if (ref $arg) {
         @yaml = <$arg>;
     }
@@ -131,41 +125,6 @@ my $deep_merge = sub {
     ) };
 };
 
-my $clone_merge = sub {
-
-    my ($left, $right) = @_;
-
-    if (reftype($left) eq 'HASH' && reftype($right) eq 'HASH')
-    {
-        my %seen;
-
-        foreach my $key (keys %$left)
-        {
-            my $clone = $left->{$key};
-
-            die "-clone of $key is not a hash\n"
-                unless reftype($clone) eq "HASH";
-
-            my $list = delete $clone->{$key};
-
-            die "-clone of $key is missing a list of keys under the same key\n"
-                unless $list && reftype($list) eq 'ARRAY';
-
-            foreach my $cloned_key (@$list)
-            {
-                die "-clone key $cloned_key must appear only once.\n"
-                    if $seen{$cloned_key}++;
-
-                $right->{$cloned_key} = $clone;
-            }
-        }
-
-        return %$right;
-    }
-
-    die "-clone must appear in a hash, and contain a hash.\n";
-};
-
 # in order of priority:
 my @SPECIAL = qw(
     -merge
@@ -173,7 +132,6 @@ my @SPECIAL = qw(
     -export
     import
     -import
-    -clone
 );
 my %SPECIAL = (
     -import => $shallow_merge,
@@ -181,82 +139,64 @@ my %SPECIAL = (
     -export => $shallow_merge,
     export  => $shallow_merge,
     -merge  => $deep_merge,
-    -clone  => $clone_merge,
 );
 
-# Note: This used to add a (heavy) dependency on Data::Visitor
-# to do these simple transformations. I *think* this is exactly 
-# equivalent to what it used to do.
-
-sub _unravel {
-    my $data = shift;
-
-    if (ref $data) {
-        $data = dclone($data) if $seen{$data}++;
-
-        if (reftype $data eq 'HASH') {
-            return _unravel_hash($data);
-        }
-        elsif (reftype $data eq 'ARRAY') {
-            for my $elt (@$data) {
-                $elt = _unravel($elt);
-            }
-            return $data;
-        }
-    }
-
-    return $data;
+{
+    package YAML::LoadBundle::YAMLDV;
+$YAML::LoadBundle::YAMLDV::VERSION = '0.001';
+use base qw(Data::Visitor::Callback);
+    # something in the way Data::Visitor tracks references it's already seen
+    # causes bizarre bugs -- it seems like memory addresses can get reused,
+    # meaning Data::Visitor grafts data into random places later down the
+    # tree.
+    # We don't have any circularity here, which is what D::V's refaddr cache
+    # exists to handle.
+    # For D::V 0.27, we could get the same result without subclassing like
+    # this:
+    #    seen => sub {
+    #        my ($v, $data, $result) = @_;
+    #        return $v->visit_no_rec_check($data);
+    #    },
+    sub visit { shift->visit_no_rec_check(@_) }
 }
 
-# Note: this modifies the argument in place. But sometimes it returns 
-# a different reference, in order to replace itself in the enclosing
-# data structure. (If it encounters a "-flatten" entry.)
-
-sub _unravel_hash {
-    my $data = shift;
-    
-    while (my @keys = grep { exists $data->{$_} } @SPECIAL) {
-        # Make sure that deeper -merges and such will be handled first
-        for my $key ( grep { ! $SPECIAL{ $_ } } keys %$data ) {
-            # False values can be skipped for performance
-            next unless $data->{$key};
-            $data->{$key} = _unravel($data->{$key})
-        }
+sub _visit_hash {
+    my ($v, $data) = @_;
+    my $result = {%$data};
 
+    while (my @keys = grep { exists $result->{$_} } @SPECIAL) {
         for my $key (@keys) {
             my $handler = $SPECIAL{$key};
-            my $val = delete $data->{$key};
+            my $val = delete $result->{$key};
             next unless $val;
-            %$data = $handler->(_unravel($val), $data);
+            %$result = $handler->($v->visit($val), $result);
         }
     }
 
-    if (keys %$data == 1) {
-        if (my $arrs = $data->{-flatten}) {
-            _unravel($arrs);
-            return [ map @$_, @$arrs ];
+    if (keys %$result == 1) {
+        if (my $arrs = $result->{-flatten}) {
+            return [ map { @{ $v->visit($_) } } @$arrs ];
         }
-        elsif (my $hrefs = $data->{-flattenhash}) {
-            _unravel($hrefs);
-            return { map %$_, @$hrefs };
+        elsif (my $hrefs = $result->{-flattenhash}) {
+            return { map { %{ $v->visit($_) } } @$hrefs };
         }
     }
 
-    for my $elt (values %$data) {
-        $elt = _unravel($elt) if ref($elt);
-    }
-
-    $data = dclone($data) if $seen{$data}++;
-    return $data;
+    return $result;
 }
 
-
 {
 my %YAML_cache;
 sub _unravel_and_cache {
     my ($path, $perl, $cache_mtime, %params) = @_;
 
-    _unravel($perl);
+    my $v = YAML::LoadBundle::YAMLDV->new(
+        hash => \&_visit_hash,
+    );
+    $perl = do {
+        local $DB::deep = 500 if defined $DB::deep;
+        $v->visit($perl);
+    };
 
     # TODO: need a better way to explicitly not cache here
     if ($cache_mtime) {
@@ -511,7 +451,7 @@ YAML::LoadBundle - Load a directory of YAML files as a bundle
 
 =head1 VERSION
 
-version v0.4.3
+version 0.001
 
 =head1 SYNOPSIS
 
@@ -521,7 +461,7 @@ version v0.4.3
 
 =head1 DESCRIPTION
 
-Adds additional features to YAML::XS to allow splitting a YAML file into
+Adds additonal features to YAML::XS to allow splitting a YAML file into
 multiple files in a common directory.
 This helps with readability when the file is large.
 
@@ -575,11 +515,7 @@ shallow merge
 
 deep merge (see L<Hash::Merge::Simple>)
 
-=item * C<-clone>
-
-clones intermediate hash keys, see L</Cloning intermediate hash keys> below.
-
-=item * C<-flatten>
+=item * <-flatten>
 
     some_key: { -flatten: [*SomeArrayRef, *SomeOtherArrayRef] }
 
@@ -613,88 +549,6 @@ Instead of a hash reference, any of these keys may contain an array reference
 of hash references, in which case those hash references are merged using
 whatever strategy normally applies (e.g. deep merge for C<-merge>).
 
-=head4 Cloning intermediate hash keys
-
-C<-clone> provides a way to repeat intermediate nodes in hashes:
-
-  apple:
-    -clone:
-       letters:
-         letters: &letters
-           - a
-           - b
-         d: e
-         f: g
-       numbers:
-         numbers:
-           - 1
-           - 2
-         quattro: 4
-         cinco:   5
-    golf: g
-    hotel: h
-
-  banana:
-    -clone:
-      letters:
-         letters:  *letters
-         h: i
-         j: k
-
-This results a hash with "d" and "f" keys getting cloned inside another
-hash using keys "a" and "b" for each cloned copy; "quattro" and "cinco"
-are also cloned under keys "1" and "2", together with "g" and "h" which are
-just along for the ride.
-
-The same "a" and "b" keys get referenced, without duplication, and used
-for intermediate keys in a different part of the hash.
-
-So the resulting hash would be:
-
-   'apple' => {
-         'a' => {
-             'd' => 'e',
-             'f' => 'g',
-          },
-         'b' => {
-             'd' => 'e',
-             'f' => 'g',
-          },
-         '1' => {
-             'quattro' => '4',
-             'cinco' => '5',
-         },
-         '2' => {
-             'quattro' => '4',
-             'cinco' => '5',
-         },
-         'golf' => 'g',
-         'hotel' => 'h',
-    },
-    'banana' => {
-         'a' => {
-             'h' => 'i',
-             'j' => 'k',
-         },
-         'b' => {
-             'h' => 'i',
-             'j' => 'k',
-         },
-     },
-
-Formally:
-
-C<-clone> itself must be a hash key, and contains a hash, the "clone" hash,
-with one or more keys. Each value of those keys is a subhash that must
-have a key with the same name as its parent key in the "clone" hash.
-This key's value must be a list, and the rest of the subhash gets cloned,
-one for each value in the list, and placed into the C<-clone> hash,
-with each value in the list being its key.
-
-This provides the means to have a single specification of a list and
-then repeat it (via the usual YAML "&" and "*") references but insert them
-as intermediate hash keys.
-
 =head3 load_yaml_bundle
 
   load_yaml_bundle($path, \%options)
@@ -842,10 +696,9 @@ Grant Street Group <developers@grantstreet.com>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2016 - 2021 by Grant Street Group.
+This software is copyright (c) 2007-2016 by Grant Street Group.
 
-This is free software, licensed under:
-
-  The Artistic License 2.0 (GPL Compatible)
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
 
 =cut
